#
# Author:: Matt Ray <matt@opscode.com>
# Cookbook Name:: zenoss
# Provider:: zenpack
#
# Copyright 2010, 2011 Opscode, Inc <legal@opscode.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'chef/mixin/shell_out'
require 'chef/mixin/language'
include Chef::Mixin::ShellOut


  # Return a list of of installed Zenpacks, using local cache when possible
  # 
  # @return [Array]  List of installed zenpacks
  def installed_zenpacks
    # Return the list from memory, if it found, otherwise refresh the list
    node.run_state[:installed_zenpacks] ||= refresh_zenpack_list()


  end
  
  
  # Refresh the list of installed Zenpacks
  #
   # @return [Hash]  of installed zenpacks, along with additional metadata
  def refresh_zenpack_list
    Chef::Log.debug("Refreshing Zenpack List")
    packs = {}
    zp_out = shell_out("sudo -u zenoss -i #{node['zenoss']['server']['zenhome']}/bin/zenpack --list")
    zps = zp_out.stdout.split("\n")
    zps.each do |zp|
      puts "*" * 60
      zp_name = zp.split(" ").first
      unless zp_name.nil?
        packs[zp_name] = {}
        
        # Try and figure out the version, for the potential of later adding
        # an upgrade action
        zp_version = zp.scan(/#{zp_name}-(.*?)-py/).first
        unless zp_version.nil? || zp_version.empty?
          packs[zp_name][:version] = zp_version.first
        end
        
        # try and determine the installation path
        zp_path = zp.scan(/\((\/.*?)\)/).first
        unless zp_path.nil? || zp_path.empty?
          packs[zp_name][:path] = zp_path.first
        end
      end
    end
    node.run_state[:installed_zenpacks] = packs
  end


action :install do
  unless @zenpack.exists
    zpfile = "#{new_resource.package}-#{new_resource.version}-#{new_resource.py_version}.egg"
    Chef::Log.info "Installing the #{zpfile} ZenPack."
    #download the ZenPack
    zpfile_path = ::File.join(Chef::Config[:file_cache_path], zpfile)
    remote_file zpfile_path do
      # TODO - Fix This: http://tickets.opscode.com/browse/COOK-3107
      source "http://dev.zenoss.com/zenpacks/#{zpfile}"
      mode "0644"
      action :create
    end
    #install the ZenPack
    execute "zenpack --install" do
      user "zenoss"
      cwd Chef::Config[:file_cache_path]
      environment ({
                     'LD_LIBRARY_PATH' => "#{node['zenoss']['server']['zenhome']}/lib",
                     'PYTHONPATH' => "#{node['zenoss']['server']['zenhome']}/lib/python",
                     'ZENHOME' => node['zenoss']['server']['zenhome']
                   })
      command "#{node['zenoss']['server']['zenhome']}/bin/zenpack --install=#{zpfile}"
      action :run
    end
    refresh_zenpack_list()
    new_resource.updated_by_last_action(true)
  end
end

action :remove do
  if @zenpack.exists
    Chef::Log.info "Removing the #{new_resource.package} ZenPack."
    execute "zenpack --remove" do
      user "zenoss"
      environment ({
                     'LD_LIBRARY_PATH' => "#{node['zenoss']['server']['zenhome']}/lib",
                     'PYTHONPATH' => "#{node['zenoss']['server']['zenhome']}/lib/python",
                     'ZENHOME' => node['zenoss']['server']['zenhome']
                   })
      command "#{node['zenoss']['server']['zenhome']}/bin/zenpack --remove=#{new_resource.package}"
      action :run
    end
    
    # Remove the pack from the cached list, rather than shelling out to rebuild
    # the list
    node.run_state[:installed_zenpacks].delete(new_resource.name)
    new_resource.updated_by_last_action(true)
  end
end

#check if the zenpack is already installed
def load_current_resource
  @zenpack = Chef::Resource::ZenossZenpack.new(new_resource.package)
  Chef::Log.debug("Checking for ZenPack #{new_resource.name}")
  exists = installed_zenpacks().has_key?(new_resource.name)
  @zenpack.exists(exists)
end